/* GSI - general sound interface
 *
 * gsi_midi.h
 *
 * Copyright (C) W.H.Scholten 1998
 *
 */

#ifndef _GSI_MIDI_H
#define _GSI_MIDI_H

#include <gsi/mytypes.h>

#define min(x,y) ((x)>(y) ? (y): (x))
#define max(x,y) ((x)<(y) ? (y): (x))

#define SEQUENCER_BUFSIZE 1024


/* synthesizer hardware available, if multiple available, use highest numbered
 * one found (GUS at the moment) by default */
#define MIDI_HW					-1
#define SYNTH_HW_NONE			0
#define SYNTH_HW_FM				1
#define SYNTH_HW_OPL			2
#define SYNTH_HW_GUS			3
#define SYNTH_HW_AWE32			4
#define SYNTH_HW_PCM_EMULATION	255


/* Standard MIDI file format definitions */
#define MThd					0x4d546864
#define MTrk					0x4d54726b

#define	MIDI_Sequence_number 	0x00
#define	MIDI_Text_event			0x01
#define MIDI_Copyright_notice 	0x02
#define MIDI_Sequence_name    	0x03
#define MIDI_Instrument_name 	0x04
#define MIDI_Lyric	        	0x05
#define MIDI_Marker				0x06
#define	MIDI_Cue_point			0x07
#define MIDI_Channel_prefix		0x20
#define	MIDI_End_of_track		0x2f
#define	MIDI_Set_tempo			0x51
#define	MIDI_Smpte_offset		0x54
#define	MIDI_Time_signature		0x58
#define	MIDI_Key_signature		0x59
#define	MIDI_Sequencer_specific	0x74
#define	MIDI_Meta_event			0xff


/* Which channels do we allow to play? */
#define IS_PLAYING(x)	(chn_mask & (1 << x))

/* Default mask for percussion instrument channels. Bit 0=channel 1, bit 9= channel 10 etc. 0x200=channel 10. */
#define PERCUSSION_CHANNELS	0x0200

/* A few convenience functions */
#define IS_PERC(x)		(perc_chn & (1 << x))

#define IS_MIDI(chn) 	(synth_types[channel_output[chn]]==MIDI_HW)
#define IS_FM(x)     	(synth_types[channel_output[chn]]==SYNTH_HW_OPL)
#define IS_GUS(x)    	(synth_types[channel_output[chn]]==SYNTH_HW_GUS)
#define IS_AWE(chn)  	(synth_types[channel_output[chn]]==SYNTH_HW_AWE32)


/* Structs for the current playing state */
struct channel_state {
	int  program;
	int  bender;
	int  bender_range;
	int  controller[128]; /* used are only 0- 0x65 it seems */
	int  pressure;
};

struct voice_state {
	int  note;
	int  channel;
	int  timestamp;
	int  available; /* playing? */
};

/* Struct representing the music data */
struct miditrack {
	byte *data;
	unsigned long length;		/* length of track (bytes) */
	unsigned long index;		/* current position in data */
	unsigned long ticks;		/* current midi tick count */
	byte running;				/* running status */
};

extern int  gus_init (int seq_fd, int dev);
extern int  gus_load (int seq_fd, int type, int dev, int pgm);
extern int  opl3_init(int seq_fd, int dev);
extern int  opl3_load (int seq_fd, int type, int dev, int pgm);
extern int  midi_to_tracks (byte *file_buf, int size);

extern void seq_reset ();
extern void seq_pause ();
extern void seq_continue ();
extern int  seq_set_patch (int chn, int pgm);
extern void seq_stop_note (int dev, int chn, int note, int vel);
extern void seq_start_note (int dev, int chn, int note, int vel);
extern void seq_key_pressure (int dev, int chn, int note, int vel);
extern void seq_control (int dev, int chn, int p1, int p2);
extern void seq_chn_pressure (int dev, int chn, int vel);
extern void seq_bender (int dev, int chn, int p1, int p2);


extern int  play_midi_init ();
extern int  play_midi_events (int max_dt);
extern void reset_song();

extern int  midi_to_tracks (byte *filebuf, int filelength);

extern double gettimeofday_diff();
extern void status_send_byte(int data);

extern int  do_callback; /* from the main server code */

extern int  seqfd;

extern int  nrsynths;
extern int  *synth_types;
extern struct synth_info *card_info;

extern int  channel_output[16]; /* which midi channel to which card? */

extern int  chn_mask; /* channels used */
extern int  perc_chn; /* channels used for percussion (default = bit 10) */
extern int  ticks; /* midi timing */
extern int  use_opl3; /* use opl 3 mode ? */

extern int  force_chn_pgm[16];

/* patch loading */
extern int  patchloaded[256]; /* replace by 8 int's */
extern int  fmloaded[256]; /* replace by 8 int's */

/* Midi tracks */
extern int  division, format;
extern int  numtracks;
extern struct miditrack *song_tracks;

/* midi playing */
extern struct voice_state **voices; /* for each sound card nrvoices */
extern double last_midi_write, current_midi; /* for midi pausing etc */

/* midi state */
extern struct channel_state channel[16];
extern int  note_vel[16][128];

/* samples, intermediate structure (data will be put in patch struct)
 * This will be used for custom patch making */
struct sample_header {
	char      name[7];
	byte      fractions;
	int32     len;
	int32     loop_start;
	int32     loop_end;
	uint16    base_freq;
	int32     low_note;
	int32     high_note;
	int32     base_note;
	int16     detune;
	byte      panning;

	byte      envelope_rate[6];
	byte      envelope_offset[6];

	byte      tremolo_sweep;
	byte      tremolo_rate;
	byte      tremolo_depth;

	byte      vibrato_sweep;
	byte      vibrato_rate;
	byte      vibrato_depth;

	char      modes;

	int16     scale_frequency;
	uint16    scale_factor;
};


#endif /* _GSI_MIDI_H*/
